@@ -1,18 +1,18 @@
 package OpenGL::Sandbox::Texture;
-BEGIN { $OpenGL::Sandbox::Texture::VERSION = '0.02_01'; } # TRIAL
-
-$OpenGL::Sandbox::Texture::VERSION = '0.0201';use Moo;
+use Moo;
 use Carp;
 use Try::Tiny;
 use OpenGL::Sandbox qw(
 	GL_TEXTURE_2D GL_TEXTURE_MIN_FILTER GL_TEXTURE_MAG_FILTER GL_TEXTURE_WRAP_S GL_TEXTURE_WRAP_T
-	glTexParameteri glBindTexture
+	glTexParameteri glBindTexture gen_textures delete_textures
 );
 use OpenGL::Sandbox::MMap;
 
 # ABSTRACT: Wrapper object for OpenGL texture
+our $VERSION = '0.049_001'; # TRIAL VERSION
 
 
+has name       => ( is => 'rw' );
 has filename   => ( is => 'rw' );
 has loader     => ( is => 'rw' );
 has loaded     => ( is => 'rw' );
@@ -39,6 +39,7 @@ sub _maybe_apply_gl_texparam {
 }
 
 
+sub _build_tx_id { gen_textures(1) }
 sub bind {
 	my ($self, $target)= @_;
 	glBindTexture($target // GL_TEXTURE_2D, $self->tx_id);
@@ -48,6 +49,11 @@ sub bind {
 	$self;
 }
 
+sub DESTROY {
+	my $self= shift;
+	delete_textures(delete $self->{tx_id}) if $self->has_tx_id;
+}
+
 
 sub load {
 	my ($self, $fname)= @_;
@@ -66,14 +72,16 @@ sub load {
 sub load_rgb {
 	my ($self, $fname)= @_;
 	my $mmap= OpenGL::Sandbox::MMap->new($fname);
-	$self->_load_rgb_square($mmap, 0);
+	$self->tx_id; # make sure it is built
+	OpenGL::Sandbox::_texture_load_rgb_square($self, $mmap, 0);
 	$self->loaded(1);
 	return $self;
 }
 sub load_bgr {
 	my ($self, $fname)= @_;
 	my $mmap= OpenGL::Sandbox::MMap->new($fname);
-	$self->_load_rgb_square($mmap, 1);
+	$self->tx_id; # make sure it is built
+	OpenGL::Sandbox::_texture_load_rgb_square($self, $mmap, 1);
 	$self->loaded(1);
 	return $self;
 }
@@ -82,8 +90,9 @@ sub load_bgr {
 sub load_png {
 	my ($self, $fname)= @_;
 	my $use_bgr= 1; # TODO: check OpenGL for optimal format
+	$self->tx_id; # make sure it is built
 	my ($imgref, $w, $h)= _load_png_data_and_rescale($fname, $use_bgr);
-	$self->_load_rgb_square($imgref, $use_bgr);
+	OpenGL::Sandbox::_texture_load_rgb_square($self, $imgref, $use_bgr);
 	$self->src_width($w);
 	$self->src_height($h);
 	$self->loaded(1);
@@ -118,8 +127,13 @@ sub _load_png_data_and_rescale {
 		or croak sprintf "$fname does not contain the expected number of data bytes (%d != %d * %d * %d)",
 			length($$dataref), $has_alpha? 4:3, $width, $height;
 	# Result is a ref to a scalar, to avoid copying
-	$dataref= _rescale_to_pow2_square($width, $height, $has_alpha, $use_bgr? 1 : 0, $dataref)
-		unless $width == $height && $width == _round_up_pow2($width);
+	unless ($width == $height && $width == OpenGL::Sandbox::_round_up_pow2($width)) {
+		$dataref= OpenGL::Sandbox::_img_rescale_to_pow2_square($width, $height, $has_alpha, $use_bgr? 1 : 0, $dataref);
+	}
+	elsif ($use_bgr) {
+		# need to swap bytes
+		OpenGL::Sandbox::_img_rgb_to_bgr($dataref, $has_alpha);
+	}
 	return $dataref, $width, $height;
 }
 
@@ -150,13 +164,6 @@ sub convert_png {
 	close $dst_fh or croak "close($dst): $!";
 }
 
-# Pull in the C file and make sure it has all the C libs available
-use OpenGL::Sandbox::Texture::Inline
-	C => do { my $x= __FILE__; $x =~ s|\.pm|\.c|; Cwd::abs_path($x) },
-	INC => '-I'.do{ my $x= __FILE__; $x =~ s|/[^/]+$|/|; Cwd::abs_path($x) }.' -I/usr/include/ffmpeg',
-	LIBS => '-lGL -lswscale',
-	CCFLAGSEX => '-Wall -g3 -Os';
-
 1;
 
 __END__
@@ -171,10 +178,14 @@ OpenGL::Sandbox::Texture - Wrapper object for OpenGL texture
 
 =head1 VERSION
 
-version 0.02_01
+version 0.049_001
 
 =head1 ATTRIBUTES
 
+=head2 name
+
+Human-readable name of this texture (not GL's integer "name")
+
 =head2 filename
 
 Path from which image data will be loaded.  If not set, the texture will not have any default
@@ -368,7 +379,7 @@ Michael Conrad <mike@nrdvana.net>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2018 by Michael Conrad.
+This software is copyright (c) 2019 by Michael Conrad.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
